<?php
namespace Repository;
/**
 * Description of Message
 *
 * @author Egrem
 */
class MessageRepository {
    
    private const ALGO = "SHA256";
    private const FIELDS = "id, sender, receiver, message, subject, is_read, send, message_hash";
    private $perPage = 6;
    
    public function save($message, $subject, $receiverId, $senderId = 0, $raw = false) : bool {
        $hash = hash(self::ALGO, $receiverId . $message);
        $existingMessage = $this->getByHash($hash);
        if( count($existingMessage) > 0 ) {
            // message was already saved
            return 1;
        }
        
        $stmt = mysqli_prepare(\Config::getDBConn(),   "INSERT INTO " . DB_PREFIX . "messages SET "
                                                    . "sender = ?, "
                                                    . "receiver = ?, "
                                                    . "message =?, "
                                                    . "subject = ?, "
                                                    . "message_hash = ?");
        if(!$raw) {
            $message = makeEntityString($message);
        }
        
        $subject = makeEntityString($subject);
        $stmt->bind_param('iisss', $senderId, $receiverId, $message, $subject, $hash);
        $stmt->execute();
        
        $ret = 0;
        // on success
        if($stmt !== false) {
            $ret = 1;
            $stmt->close();
        }
        return $ret;
    }
    
    public function getAllForUser($uId, $site = 1) {
        $result = [];
        $startPos = ($site -1) * $this->perPage;
        $stmt = mysqli_prepare(\Config::getDBConn(), "SELECT " . self::FIELDS ." "
                . "FROM " . DB_PREFIX . "messages "
                . "WHERE receiver = ? "
                . "ORDER BY send DESC "
                . "LIMIT ?, ?");
        $stmt->bind_param('iii', $uId, $startPos, $this->perPage);
        $stmt->execute();
        
        if ( $stmt === false) {
            return [];
        }
        
        $res = $stmt->get_result();
        while($row = $res->fetch_assoc()) {
            $result [] = $this->makeBaseResult($row);
        }
        return $result;
    }
    
    public function getCountForUser($uId, $onlyNew = false) : int {
        $result = 0;
        
        $new = "";
        if( $onlyNew ) {
            $new = " and is_read = 0";
        }
        $stmt = mysqli_prepare(\Config::getDBConn(), "SELECT COUNT(id) as count "
                . "FROM " . DB_PREFIX . "messages "
                . "WHERE receiver = ?" . $new);
        $stmt->bind_param('i', $uId);
        $stmt->execute();
        
        if ( $stmt === false) {
            return 0;
        }
        
        $res = $stmt->get_result();
        if ( $res->num_rows === 1 ) {
            $row = $res->fetch_assoc();
            $result = $row['count'];
        }
        return $result;
    }
    
    public function getByHash($hash) : array {
        $result = [];
        $stmt = mysqli_prepare(\Config::getDBConn(), "SELECT " . self::FIELDS ." FROM " . DB_PREFIX . "messages WHERE message_hash = ?");
        $stmt->bind_param('s', $hash);
        $stmt->execute();
        
        if ( $stmt === false) {
            return [];
        }
        
        $res = $stmt->get_result();

        // hash is unique therefore we can expect 1 or 0 results
        if ( $res->num_rows === 1 ) {
            $row = $res->fetch_assoc();
            $result = $this->makeBaseResult($row);
        }
        $stmt->close();
        
        return $result;
    }
    
    public function get($id) : array {
        $result = [];
        $stmt = mysqli_prepare(\Config::getDBConn(), "SELECT " . self::FIELDS .", u.nick as sender_nick "
                . "FROM " . DB_PREFIX . "messages m "
                . "LEFT JOIN " . DB_PREFIX . "users u ON u.uid=m.sender "
                . "WHERE id = ?");
        $stmt->bind_param('i', $id);
        $stmt->execute();
        
        if ( $stmt === false) {
            return [];
        }
        
        $res = $stmt->get_result();

        // hash is unique therefore we can expect 1 or 0 results
        if ( $res->num_rows === 1 ) {
            $row = $res->fetch_assoc();
            $baseRes = $this->makeBaseResult($row);
            $addRes = [
                'senderNick' => ($row['sender_nick'] == null ? 'System' : $row['sender_nick']),
            ];
            $result = array_merge($baseRes, $addRes);
        }
        $stmt->close();
        
        return $result;
    }
    
    public function setAsRead($id) : bool {
        $stmt = mysqli_prepare(\Config::getDBConn(),  "UPDATE " . DB_PREFIX . "messages SET "
                                                    . "is_read = 1 "
                                                    . "WHERE id = ?");
        
        $stmt->bind_param('i', $id);
        $stmt->execute();
        
        $ret = 0;
        // on success
        if($stmt !== false) {
            $ret = 1;
            $stmt->close();
        }
        return $ret;
    }

    /**
     * Delete old and read messages
     * @param int $userId The id of the user
     * @param int $days number of days how long messages will be kept (default: 30)
     * @return bool
     */
    public function deleteOld($userId, $days = 30) : bool {
        $stmt = mysqli_prepare(\Config::getDBConn(),  "DELETE FROM " . DB_PREFIX . "messages "
                                                    . "WHERE receiver = ? AND DATEDIFF(CURDATE(), send) > ? AND is_read = 1");
        
        $stmt->bind_param('ii', $userId, $days);
        $stmt->execute();

        $ret = 0;
        // on success
        if($stmt !== false) {
            $ret = 1;
            $stmt->close();
        }
        return $ret;
    }
        
    private function makeBaseResult($row) : array {
        if(!is_array($row)) {
            return [];                
        }
        return [
                'id' => $row['id'],
                'sender' => $row['sender'],
                'receiver' => $row['receiver'],
                'message' => $row['message'],
                'subject' => $row['subject'],
                'isRead' => $row['is_read'],
                'hash' => $row['message_hash'],
                'send' => new \DateTime($row['send'])
            ];
    }
    
    public function calculatePages($number)
    {
        $page = 1;
        $page = $number / $this->perPage;	//anzahl der beitrge durch eintrge pro seite -- ergebnis z.b. 1.4
        $page = ceil($page);			//Aufrunden der Komma Zahl

        return $page;
    }
}
