<?php

/**
 * This class handles all the security stuff like password handling
 *
 * @author Egrem
 */
class Security {

    /**
     * Generates a new password hash based on the given string
     * @param string $pw
     * @return string
     */
    public function hashPssword($pw) {
        $algo = PASSWORD_DEFAULT; // fallback hash algo
        if (defined('PASSWORD_ARGON2ID')) {
            $algo = PASSWORD_ARGON2ID;
        } else if (defined('PASSWORD_ARGON2I')) {
            $algo = PASSWORD_ARGON2I;
        }
        return password_hash($pw, $algo);
    }

    /**
     * Checks whether the given password matches the stored one
     * It has a fallback when a hash is used based on the old sha1 algo
     * If the password matches it will update the hash with the new algo in the DB
     * @param int $uId
     * @param string $pw
     * @param string $hash
     * @return boolean
     */
    public function verifyPassword($uId, $pw, $hash) {
        // for the old way, sha1 does not start with "$"
        if (!preg_match('/^\$.*/', $hash)) {
            $password = sha1(utf8_decode($pw));
            $result = hash_equals($hash, $password);
            // alwys update the stored hash with the new algo
            if ($result) {
                $this->updatePassword($uId, $pw);
            }
            return $result;
        } else {
            return password_verify(utf8_decode($pw), $hash);
        }
    }

    /**
     * Verify the login
     * @param string $name
     * @param string $password
     * @param string $permanent
     * @return boolean
     */
    public function verifyLogin($name, $password, $permanent) {
        $result = false;
        $name = utf8_decode($name);
        $stmt = mysqli_prepare(Config::getDBConn(), "SELECT uid, nick, pwd, titel, mail FROM " . DB_PREFIX . "users WHERE bot=0 and nick LIKE ?");
        $stmt->bind_param('s', $name);
        $stmt->execute();
        $res = $stmt->get_result();

        if ($res->num_rows === 1) {
            $row = $res->fetch_assoc();
            if ($this->verifyPassword($row['uid'], $password, $row['pwd'])) {
                $result = true;
                // protection for session fixation
                session_regenerate_id();
                $_SESSION['login'] = true;
                $_SESSION['uid'] = $row['uid'];
                $_SESSION['nick'] = $row['nick'];
                $_SESSION['titel'] = $row['titel'];
                $_SESSION['mail'] = $row['mail'];

                if ($permanent == 1) {
                    $this->permanentLogin();
                }
                //$this->setLastAction();
            }
        }
        $stmt->close();
        if (!$result) {
            // add some protection for user enumeration attacks via site channel 
            // when the user is not found and password_hash not called which costs some extra time
            usleep(random_int(100000, 500000));
        }

        return $result;
    }

    /**
     * Update the user's password in the DB
     * @param int $uId
     * @param string $pw
     */
    public function updatePassword($uId, $pw) {
        $password = $this->hashPssword($pw);
        $stmt = mysqli_prepare(Config::getDBConn(), "UPDATE " . DB_PREFIX . "users SET pwd= ? WHERE uid= ?");
        $stmt->bind_param('si', $password, $uId);
        $stmt->execute();
        $stmt->close();
    }

    /**
     * Perform the logout and delete session and cookies
     */
    public function doLogout() {
        session_regenerate_id(true);
        session_destroy();
        if (isset($_COOKIE['login1']) || isset($_COOKIE['login2'])) {  //Cookies lschen
            $_COOKIE['login1'] = "";
            $_COOKIE['login2'] = "";
            setcookie('login1');
            setcookie('login2');
        }
    }

    /**
     * Add cookies for permanent login
     */
    private function permanentLogin() {
        $uId = $_SESSION['uid'] ?? 0;
        if ($uId > 0) {
            $values = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
            $cookieSession = "";
            for ($i = 0; $i < 20; $i++) {
                $pos = rand(0, 61);
                $cookieSession .= $values[$pos];
            }
            $currentCookieParams = session_get_cookie_params();
            $path = dirname($_SERVER['PHP_SELF']);
            $domain = $currentCookieParams["domain"];
            $ssl = self::isSSL();
            setcookie('login1', $_SESSION['nick'], time() + 3600 * 24 * 100, $path, $domain, $ssl, true);
            setcookie('login2', $cookieSession, time() + 3600 * 24 * 100, $path, $domain, $ssl, true);
            $stmt = mysqli_prepare(Config::getDBConn(), "UPDATE " . DB_PREFIX . "users SET cookie= ? WHERE uid= ?");
            $stmt->bind_param('si', $cookieSession, $uId);
            $stmt->execute();
            $stmt->close();
        }
    }

    public static function setSessionAndCookieAttributes(string $dir) {
        $currentCookieParams = session_get_cookie_params();
        session_set_cookie_params(
                $currentCookieParams["lifetime"],
                $dir, //dirname($_SERVER['PHP_SELF']),
                $currentCookieParams["domain"],
                self::isSSL(), // secure
                true    // httponly
        );
        session_name('sid'); 
    }

    public static function isSSL(): bool {
        // IIS will return on or off
        if (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off') {
            return true;
        }

        // when ports are forwarded like behind a reverse proxy
        if (!empty($_SERVER['HTTP_X_FORWARDED_PROTO']) && $_SERVER['HTTP_X_FORWARDED_PROTO'] === 'https') {
            return true;
        }

        return false;
    }

}
